//
//  main.c
//
//  Wall Clock 0.1
//  Copyright (C) 2010 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		Circle dot display clock.
//		Auto adjust by radio wave JJY.
//
//	Setting
//		PSoC1
//			Device	: CY8C27443
//			IDE		: PSoC Designer 5.0 SP6 - Build 1127 - 30-October-2009.14:48:28
//			Compiler: Image Craft C Compiler 7.02.004
//
//
//	Update history
//	---------- ----- -------------------------------------------
//	2010.07.01 v0.1  First release
//
//


#include <string.h>

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include "PSoCGPIOINT.h"

#include "t_sprintf.h"

// Setup
//#define LCD_ENABLE						// Status display for DEBUG
//#define LED_ENABLE						// Status display for DEBUG

// Constant
#define MIN(a,b)	((a)<(b) ? (a) : (b))
#define MAX(a,b)	((a)>(b) ? (a) : (b))

#define JJY_STAT_ABORT	-1	// Inactive
#define JJY_STAT_NORMAL	0	// Inactive
#define JJY_STAT_ADJUST	1	// Active
#define JJY_STAT_CANCEL	2	// Active

//
// Global area
//
char	buf[16];

//
// Local area
//
#define AGC_TBL_SZ	16
const struct {
	BYTE pga1_gain;
	BYTE ampinv1_gain;
} agc_tbl[AGC_TBL_SZ] = {
	{PGA_1_G16_0, AMPINV_1_G47_0},	// x 752
	{PGA_1_G24_0, AMPINV_1_G23_0},	// x 552
	{PGA_1_G16_0, AMPINV_1_G23_0},	// x 368
	{PGA_1_G16_0, AMPINV_1_G15_0},	// x 240
	{PGA_1_G24_0, AMPINV_1_G7_00},	// x 168
	{PGA_1_G16_0, AMPINV_1_G7_00},	// x 112
	{PGA_1_G16_0, AMPINV_1_G4_33},	// x 70
	{PGA_1_G16_0, AMPINV_1_G3_00},	// x 48
	{PGA_1_G8_00, AMPINV_1_G4_33},	// x 34.6
	{PGA_1_G3_20, AMPINV_1_G7_00},	// x 22.4
	{PGA_1_G2_00, AMPINV_1_G7_00},	// x 14.0
	{PGA_1_G3_20, AMPINV_1_G3_00},	// x 9.6
	{PGA_1_G2_67, AMPINV_1_G2_20},	// x 5.9
	{PGA_1_G2_00, AMPINV_1_G2_20},	// x 4.4
	{PGA_1_G1_60, AMPINV_1_G1_67},	// x 2.67
	{PGA_1_G2_00, AMPINV_1_G1_00}	// x 2.0
};

#define SIG_TYPE_MARKER		9
#define SIG_TYPE_UNKNOWN	0xff
const struct {
	unsigned char	patt;
	unsigned char	val;
}	wave_to_sig[] = {
	{0b00000001, SIG_TYPE_MARKER},
	{0b10000001, SIG_TYPE_MARKER},
	{0b11000001, 1},
	{0b11100001, 1},
	{0b11110001, 1},
	{0b11111001, 0},
	{0b11111101, 0},
//	{0b01000001, SIG_TYPE_MARKER},
//	{0b01110001, 1},
//	{0b01111101, 0},
//	{0b01100001, 1},
//	{0b01111001, 0},
	{0,          SIG_TYPE_UNKNOWN}	// Terminater
};

const BYTE	msg01[] = {
	// gfedcba
	0b01101101,		// s
	0b01101110,		// y
	0b01010100,		// n
	0b01011000,		// c
	0b00011100,		// u
	0b01010100,		// n
	0b00000000,		// -
	0b00000000		// -
};

//-----------------------------------------------------------------------------
//
//	Local subroutines
//
//-----------------------------------------------------------------------------
//
// Calculate parity
//
unsigned char calc_parity(unsigned char a){
	unsigned char i;
	unsigned char j;
	
	j = 0;
	for(i = 8; i > 0; --i) {
		if (a & 0b00000001) {
			j++;
		}
		a >>= 1;
	}
	return(j & 0b00000001);
}

//
// I2C 7-segment LED Control
//
#define I2C_LED7SEG_SAD		0x07			// I2C Slave address
#define I2C_LED7SEG_Digit1	0x00
#define I2C_LED7SEG_Digit2	0x01
#define I2C_LED7SEG_Digit3	0x02
#define I2C_LED7SEG_Digit4	0x03
#define I2C_LED7SEG_Digit5	0x04
#define I2C_LED7SEG_Digit6	0x05
#define I2C_LED7SEG_Digit7	0x06
#define I2C_LED7SEG_Digit8	0x07
#define I2C_LED7SEG_DimOff	0x00
#define I2C_LED7SEG_DimOn	0x01
#define I2C_LED7SEG_DimOn1	0x01
#define I2C_LED7SEG_DimOn2	0x02
#define I2C_LED7SEG_DimOn3	0x03
#define I2C_LED7SEG_DimStart	0xfe
#define I2C_LED7SEG_DimStop	0xff
#define I2C_LED7SEG_DpOn	0x01
#define I2C_LED7SEG_DpOff	0x00
BYTE i2c_led7seg_cmd01[] = {
	0x00,	         					// Command byte: Set display data
	0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x07,0x00
//	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
BYTE i2c_led7seg_cmd02[1];				// Command byte
const unsigned char	i2c_led7seg_font[] = {
	//.gfedcba
	0b00111111,		// 0
	0b00000110,		// 1
	0b01011011,		// 2
	0b01001111,		// 3
	0b01100110,		// 4
	0b01101101,		// 5
	0b01111101,		// 6
	0b00000111,		// 7
	0b01111111,		// 8
	0b01101111,		// 9
	0b01110111,		// A
	0b01111100,		// b
	0b01011000,		// c
	0b01011110,		// d
	0b01111001,		// E
	0b01110001		// F
};
void I2C_LED7SEG_Start(void){
	I2Cm_1_Start();
	return;
}
void I2C_LED7SEG_DispInt(int iValue, BYTE bPos, BYTE bLSD){
	bPos += bLSD;
	for(; bLSD > 0; --bLSD) {
		unsigned char	a;
		
		a = iValue % 10;
		iValue /= 10;
		i2c_led7seg_cmd01[bPos] = i2c_led7seg_font[a];
		--bPos;
	}
	I2Cm_1_bWriteBytes(I2C_LED7SEG_SAD, i2c_led7seg_cmd01, sizeof(i2c_led7seg_cmd01), I2Cm_1_CompleteXfer);
	return;
}
void  I2C_LED7SEG_DP(BYTE bOnOff, BYTE bDigit) {
	if (bOnOff) {
		i2c_led7seg_cmd01[bDigit + 1] |= 0x80;
	} else {
		i2c_led7seg_cmd01[bDigit + 1] &= 0x7f;
	}
	I2Cm_1_bWriteBytes(I2C_LED7SEG_SAD, i2c_led7seg_cmd01, sizeof(i2c_led7seg_cmd01), I2Cm_1_CompleteXfer);
}
void I2C_LED7SEG_PutPattern(BYTE bPattern, BYTE bDigit){
	i2c_led7seg_cmd01[bDigit + 1] = bPattern;
	I2Cm_1_bWriteBytes(I2C_LED7SEG_SAD, i2c_led7seg_cmd01, sizeof(i2c_led7seg_cmd01), I2Cm_1_CompleteXfer);
	return;
}
void I2C_LED7SEG_Dim(BYTE bDim){
	switch(bDim){
	case I2C_LED7SEG_DimStop:
		i2c_led7seg_cmd02[0] = 0x20;
		break;
	case I2C_LED7SEG_DimStart:
		i2c_led7seg_cmd02[0] = 0x21;
		break;
	default:
		i2c_led7seg_cmd02[0] = 0x10 | bDim;
		break;
	}
	I2Cm_1_bWriteBytes(I2C_LED7SEG_SAD, i2c_led7seg_cmd02, sizeof(i2c_led7seg_cmd02), I2Cm_1_CompleteXfer);
	return;
}
void I2C_LED7SEG_Stop(void){
	I2Cm_1_Stop();
	return;
}

//
// I2C 60dot LED Control
//
#define I2C_LED60_SAD		0x09		// I2C Slave address
#define I2C_LED60_DimOff	0x00
#define I2C_LED60_DimOn		0x01
#define I2C_LED60_DimOn1	0x01
#define I2C_LED60_DimOn2	0x02
#define I2C_LED60_DimOn3	0x03
#define I2C_LED60_DimStart	0xfe
#define I2C_LED60_DimStop	0xff
BYTE i2c_led60_cmd01[] = {
	0x00,	         					// Command byte: Set display data
	0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
BYTE i2c_led60_cmd02[1];				// Command byte

void I2C_LED60_Start(void){
	I2Cm_1_Start();
	return;
}
void I2C_LED60_DrawBG(signed char bPos, signed char bLen, signed char bPixelColEnd) {
	unsigned char	r;					// Reverse
	unsigned char	n;					// No I/O
	
	if (bPixelColEnd < 0) {
		bPixelColEnd = -bPixelColEnd;
		n = 1;
	} else {
		n = 0;
	}
	if (bLen < 0) {
		bLen = bPixelColEnd + bLen;
		r = 1;
	} else {
		r = 0;
	}
	for(; bPos < 0; bPos += 60) {
		;
	}
	for( ; bPixelColEnd > 0; --bPixelColEnd, bPos++) {
		unsigned char	o;
		unsigned char	i;
		unsigned char	b;
		unsigned char	f;
		
		for(; bPos >= 60; bPos -= 60) {
			;
		}
		o = bPos;
		if (bPos >= 15) {
			o++;
		}
		if (bPos >= 30) {
			o++;
		}
		if (bPos >= 45) {
			o++;
		}
		i = (o >> 3) + 1;
		b = 1 << (o & 0x07);

		if (bLen > 0) {
			--bLen;
			f = 1;
		} else {
			f = 0;
		}

		if (f ^ r) {
			i2c_led60_cmd01[i] |= b;
		} else {
			i2c_led60_cmd01[i] &= ~b;
		}
	}
	if (!n) {
		I2Cm_1_bWriteBytes(I2C_LED60_SAD, i2c_led60_cmd01, sizeof(i2c_led60_cmd01), I2Cm_1_CompleteXfer);
	}
	return;
}
void I2C_LED60_Dim(BYTE bDim){
	switch(bDim){
	case I2C_LED7SEG_DimStop:
		i2c_led60_cmd02[0] = 0x20;
		break;
	case I2C_LED7SEG_DimStart:
		i2c_led60_cmd02[0] = 0x21;
		break;
	default:
		i2c_led60_cmd02[0] = 0x10 | bDim;
		break;
	}
	I2Cm_1_bWriteBytes(I2C_LED60_SAD, i2c_led60_cmd02, sizeof(i2c_led60_cmd02), I2Cm_1_CompleteXfer);
	return;
}
void I2C_LED60_Stop(void){
	I2Cm_1_Stop();
	return;
}

//-----------------------------------------------------------------------------
//
//	1 sec interval	<ISR>
//
//-----------------------------------------------------------------------------
static char ddom[] = {0, 31,28,31,30,31,30,31,31,30,31,30,31};
static char yy = 10;
static char mm =  1;
static char dd =  1;
static char hh =  0;
static char mi =  0;
static char ss =  0;

#pragma interrupt_handler Timer16_1_ISR
void Timer16_1_ISR(void) {
//void clock_count_up(void) {
	ss++;
	if (ss > 59) {
		ss = 0;
		mi++;
		if (mi > 59) {
			mi = 0;
			hh++;
			if (hh > 23) {
				hh = 0;
				dd++;
				if (((yy & 0x03) == 0) && ((yy % 100) != 0)) {
					ddom[2] = 29;
				} else {
					ddom[2] = 28;
				}
				if (dd > ddom[mm]) {
					dd = 1;
					mm++;
					if (mm > 12) {
						mm = 1;
						yy++;
					}
				}
			}
		}
	}
	return;
}

//-----------------------------------------------------------------------------
//
//	Date Time key input & modify
//
//-----------------------------------------------------------------------------
void dateset(void) {
	signed char y,m,d,h,n,dom;
	char p;

	LCD_1_Control(LCD_1_CURSOR_BLINK);
	for(p = 0; p < 16; p++) {
		I2C_LED7SEG_PutPattern(0, p);
	}

	y = yy;
	m = mm;
	d = dd;
	h = hh;
	n = mi;
	p = 1;								// Change item#

	dom = ddom[m];

	for(;;) {
		char r,c;						// LCD Row, Column
		BYTE l;							// 7SEG Label
		char v;							// 7SEG Value

		// display
		switch(p) {
		case 1:							// yy
			r = 0;
			c = 1;
			l = 0b01101110;
			v = y;
			break;
		case 2:							// mm
			r = 0;
			c = 4;
			l = 0b00110111;
			v = m;
			break;
		case 3:							// dd
			r = 0;
			c = 7;
			l = 0b01011110;
			v = d;
			break;
		case 4:							// hh
			r = 1;
			c = 1;
			l = 0b01110100;
			v = h;
			break;
		default:						// mi
			r = 1;
			c = 4;
			l = 0b01010100;
			v = n;
			break;
		}
		t_sprintf(buf, "%02d.%02d.%02d", y, m, d);
		LCD_1_Position(0, 0);
		LCD_1_PrString(buf);
		t_sprintf(buf, "%02d:%02d:00", h, n);
		LCD_1_Position(1, 0);
		LCD_1_PrString(buf);
		LCD_1_Position(r, c);

		I2C_LED7SEG_PutPattern(l, I2C_LED7SEG_Digit5);
		I2C_LED7SEG_PutPattern(l, I2C_LED7SEG_Digit6);
		I2C_LED7SEG_DispInt(v, I2C_LED7SEG_Digit1, 2);

		
		// Key off wait
		while (!(SW_Up_Data_ADDR     & SW_Up_MASK));
		while (!(SW_Down_Data_ADDR   & SW_Down_MASK));
		while (!(SW_Select_Data_ADDR & SW_Select_MASK));

		// Key push wait
		while ((SW_Up_Data_ADDR     & SW_Up_MASK)    &&
		       (SW_Down_Data_ADDR   & SW_Down_MASK)  &&
		       (SW_Select_Data_ADDR & SW_Select_MASK) );

		// Key push
		if (!(SW_Select_Data_ADDR & SW_Select_MASK)) {
			p++;
			if (p > 5) break;					// forever exit
		}
		if (!(SW_Up_Data_ADDR & SW_Up_MASK)) {
			switch(p) {
			case 1:
				y++;
				y = (y > 99) ? 0 : y;
				break;
			case 2:
				m++;
				m = (m > 12) ? 1 : m;
				break;
			case 3:
				d++;
				d = (d > dom) ? 1 : d;
				break;
			case 4:
				h++;
				h = (h > 23) ? 0 : h;
				break;
			default:
				n++;
				n = (n > 60) ? 0 : n;
				break;
			}
		}
		if (!(SW_Down_Data_ADDR & SW_Down_MASK)) {
			switch(p) {
			case 1:
				--y;
				y = (y < 0) ? 99 : y;
				break;
			case 2:
				--m;
				m = (m < 1) ? 12 : m;
				break;
			case 3:
				--d;
				d = (d < 1) ? dom : d;
				break;
			case 4:
				--h;
				h = (h < 0) ? 23 : h;
				break;
			default:
				--n;
				n = (n < 0) ? 59 : n;
				break;
			}
		}
		
		// Chattering cancel
		LCD_1_Delay50uTimes(200);
	}
	
	// Fix
	yy = y;
	mm = m;
	dd = d;
	hh = h;
	mi = n;
	ss = 0;
	LCD_1_Control(LCD_1_DISP_ON);
	
	return;
}

//-----------------------------------------------------------------------------
//
//	Recieve JJY task
//
//-----------------------------------------------------------------------------
signed char recv_jjy(signed char stat) {
#define FIFO_BUF_SZ		10
#define MAX_FRAME_NUM	6
#define RETRY_NUM		3
	static unsigned char	init_flg = 0;
	static unsigned char	samp_cnt;
	static unsigned short	cd[MAX_FRAME_NUM];		// 1 Cycle data
	static unsigned short	ce[MAX_FRAME_NUM];		// error bit
	static signed char		fifo_buf[FIFO_BUF_SZ];
	static unsigned char	fifo_ep;
	static signed char		retry_ctr;
	static signed char		agc_delay;
	static short			agc_sum;
	signed char				ad;					// A/D converter output

	//
	// Open user modules
	//
	if (!init_flg) {
		unsigned char i;
		
		init_flg = 1;
		
		// Display off (reducing noise)
		I2C_LED7SEG_Dim(I2C_LED7SEG_DimStop);
		I2C_LED60_Dim(I2C_LED60_DimStop);

		// Analog power on
		AN_Power_Start();
		AN_Power_On();
		
		// Internal clock for analog block
		Counter16_1_Start();				// Frequency convert clock, output BC0(broadcast clock #0)
		Counter8_1_Start();					// BPF4_1 clock to ACLK0(analog clock #0)
		Counter16_2_Start();				// LPF2_1 clock to ACLK1(analog clock #1)

		// 1st amplifire (Cypress application note: AN2320)
		PGA_1_Start(PGA_1_HIGHPOWER);
		AMPINV_1_Start(AMPINV_1_HIGHPOWER);

		// Heterodyne convert frequency and 1st filter (Cypress application note: AN2111)
		BPF4_1_Start(BPF4_1_HIGHPOWER);		// Moduration by BC0

		// 2nd amplifire
		RefMux_1_Start(RefMux_1_HIGHPOWER);
		PGA_2_Start(PGA_2_HIGHPOWER);

		// FullWave rectifier and ripple filter (Cypress application note: AN2044 and example document No.001-58437)
		ACB02CR1 |= 0b01000000;				// <Special> Col2_ACB(PGA2)->Compbus2 output
		AMD_CR0 |= 0b01100000;				// <Special> Signal modulation: Col2_ASC(LPF2_1)<-Compbus2(PGA_2)
		LPF2_1_Start(LPF2_1_HIGHPOWER);
		
		// etc
		SAR6_1_Start(SAR6_1_HIGHPOWER);
		LED_1_Start();

		// Init
		samp_cnt = 0;
		memset((void *)cd, 0, sizeof(cd));
		memset((void *)ce, 0xff, sizeof(ce));
		memset((void *)fifo_buf, 0, sizeof(fifo_buf));
		fifo_ep	  = FIFO_BUF_SZ - 1;
		agc_delay = 0;
		agc_sum   = 0;
		retry_ctr = RETRY_NUM;

		// Debug info
#ifdef LCD_ENABLE
		LCD_1_Position(0, 8);
		LCD_1_PrCString("  wait..");
#endif
	}
	
	//
	// Measure
	//
	ad = SAR6_1_cGetSample();
#ifdef LCD_ENABLE
	LCD_1_DrawBG(1, 10, 6, ad);
#endif
#ifdef LED_ENABLE
	I2C_LED60_DrawBG(45, ad >> 1, 15);
#endif

	//
	// Entery to the FIFO buffer
	//
	fifo_ep++;
	if (fifo_ep >= FIFO_BUF_SZ) {
		fifo_ep = 0;
	}
	fifo_buf[fifo_ep] = ad;
	samp_cnt++;
						
	//
	// AGC(automatic gain control) sub task
	//
	if (--agc_delay > 0) {				// Skip for feedback time
		agc_sum += ad;
	} else {
#define AGC_LEVEL_H	24					// Wide setting
#define AGC_LEVEL_L	8
//#define AGC_LEVEL_H	22				// Standard
//#define AGC_LEVEL_L	10
//#define AGC_LEVEL_H	26				// Standard/High-shift
//#define AGC_LEVEL_L	14
//#define AGC_LEVEL_H	20				// Narrow setting
//#define AGC_LEVEL_L	12
		static unsigned char	agc_tbl_p = 3;
		signed char				ave;

		// The average of the signal level
		ave = agc_sum >> 6;
				
		// Reset counter
		agc_delay = 64;
		agc_sum = 0;
		
		// Increase and decrease gain to the signal average
		{
			char	f;
					
			f = 0;
			if (ave > AGC_LEVEL_H) {
				if (agc_tbl_p < (AGC_TBL_SZ-1)) {
					agc_tbl_p++;
					f = 1;
				}
			}
			if (ave < AGC_LEVEL_L) {
				if (agc_tbl_p > 0) {
					--agc_tbl_p;
					f = 1;
				}
			}
				
			// The amplifire gain is changed
			if (f) {
				PGA_1_SetGain(agc_tbl[agc_tbl_p].pga1_gain);
				AMPINV_1_SetGain(agc_tbl[agc_tbl_p].ampinv1_gain);
			}
		}
				
		// Debug info
#ifdef LCD_ENABLE
		LCD_1_Position(1, 9);
		LCD_1_WriteData(agc_tbl_p + '0');
#endif
#ifdef LED_ENABLE
		I2C_LED60_DrawBG(0, agc_tbl_p, 15);
#endif
	}

	//
	// Decode sub task
	//
	{
		static unsigned char	sig_patt = 0;	// Last 8bit patturn
		unsigned char			sig_flag;		// Now bit
		
		// Binarise
		{
			char		i;
			signed char	n;			// min
			signed char	x;			// max
			signed char	c;			// center
			signed char	o;			// offset
			signed char	l;			// low 
			signed char	h;			// h
					
			n = 127;
			x = 0;
			for(i = 0; i < FIFO_BUF_SZ; i++) {
				n = MIN(n, fifo_buf[i]);
				x = MAX(x, fifo_buf[i]);
			}
			c = (x + n) >> 1;
			o = (x - n) >> 2;
			l = c - o;
			h = c + o;
			
			if ((sig_patt & 0b00000001)) {	// Before signal
				sig_flag = (ad <= l) ? 0 : 1;
			} else {
				sig_flag = (ad >= h) ? 1 : 0;
			}
			LED_1_Switch(sig_flag);

			sig_patt <<= 1;
			sig_patt |= sig_flag;
		}

		// Wave rise point
		if (samp_cnt > 8) {					// Timeout
			sig_patt = (sig_patt & 0b11111100) | 0b00000001;	// insert rise point
		}
		if ((sig_patt & 0b00000011) == 0b00000001) {
			static unsigned short	fr = 0;		// Frame data, ="10sec data" ="marker terminated signal"
			static unsigned short	er = 0;		// Error bit
			static unsigned short	bn = 9;		// Bit(signal) number [0..8]
			static unsigned short	fn = 6;		// Frame number [0..5]
			char	st;					// Signal type

			samp_cnt = 0;
			
			// Signal type is juged
			{
				char	i;
					
				for(i = 0; wave_to_sig[i].val != SIG_TYPE_UNKNOWN; i++) {
					if (wave_to_sig[i].patt == sig_patt) break;
				}
				st = wave_to_sig[i].val;

				// Debug info
				if (fn < 6) {
					char	c;
					
					switch(st) {
					case 0:					c = '0';	break;
					case 1:					c = '1';	break;
					case SIG_TYPE_MARKER:	c = '*';	break;
					default:				c = '?';
//						LCD_1_Position(0, 0);
//						LCD_1_PrHexByte(sig_patt);
						break;
					}
#ifdef LCD_ENABLE
					LCD_1_Position(0, fn+10);
					LCD_1_WriteData(c);
#endif
				}
			}
				
			// Assembly frame(data... marker) data
			switch(st) {
			case 0:
			case 1:
				// Append to frame
				fr <<= 1;
				fr |= st;
				er <<= 1;
				bn++;
				break;
			case SIG_TYPE_UNKNOWN:
				// Frame error bit on
				fr <<= 1;
				er <<= 1;
				er |= 1;
				bn++;
				break;
			default:
				break;
			}

			// Frame close
			if ((st == SIG_TYPE_MARKER) || (bn > 9)) {

				// Marker process
				if (bn == 0) {		// Empty frame
						
					// Start next 1 cycle (1min)
					fn = 0;

				// Position marker process
				} else {

					// Frame format check
					if (((fn == 0) && (bn != 8)) ||		// Length
					    ((fn != 0) && (bn != 9))) {
						er = 0xffff;					// All bit is invalid
					}
						
					// Move frame data to cycle data
					if (fn < 6) {
						if (!er) {
							cd[fn] = (fr & ~er) | (cd[fn] & er);
							ce[fn] = er & ce[fn];
						}

						// Debug info
						{
							char	c;
					
							if (ce[fn]) {
								c = 'X';
							} else {
								c = '\377';	// Box
							}
#ifdef LCD_ENABLE
							LCD_1_Position(0, fn+10);
							LCD_1_WriteData(c);
#endif
#ifdef LED_ENABLE
							I2C_LED60_DrawBG(15, fn << 1, 15);
#endif
						}
					}

					// Frame clear
					fn++;
					bn = 0;
					fr = 0;
					er = 0;
				}

				// Complete 1 cycle
				if (fn == 6) {
					short	ddd;
					char	y;
					char	m;
					char	d;
					char	h;
					char	n;
					unsigned char	pa1;
					unsigned char	pa2;
					unsigned char	i;
						
					// Data convert & check
					for(i = 0; i < 6; i++) {
						if (ce[i] != 0) {
							break;
						}
					}
					n   = (((unsigned char)cd[0] & 0xe0) >> 5) * 10 + ((unsigned char)cd[0] & 0xf);
					h   = (((unsigned char)cd[1] & 0x60) >> 5) * 10 + ((unsigned char)cd[1] & 0xf);
					ddd = ((cd[2] & 0x0060) >> 5) * 100 + (cd[2] & 0x000f) * 10 + ((cd[3] & 0x01e0) >> 5);
					y   = (((unsigned char)cd[4] & 0xf0) >> 4) * 10 + ((unsigned char)cd[4] & 0xf);
					if (((y & 0x03) == 0) && ((y % 100) != 0)) {
						ddom[2] = 29;
					} else {
						ddom[2] = 28;
					}
					for(m = 1; m <= 12; m++) {
						if (ddd <= ddom[m]) {
							d = ddd;
							break;
						}
						ddd -= ddom[m];
					}
					pa1 = (((unsigned char)cd[3] >> 2) & 0b00000001) == calc_parity((unsigned char)cd[1]);
					pa2 = (((unsigned char)cd[3] >> 1) & 0b00000001) == calc_parity((unsigned char)cd[0]);

					// No error
					if ((i == 6) && pa1 && pa2) {

						// Set time
						yy = y;
						mm = m;
						dd = d;
						mi = n;
						hh = h;
						ss = 59;

						// Leave from main loop
						stat = JJY_STAT_NORMAL;
//						bn = 9;
//						fn = 99;

						// Debug info
#ifdef LCD_ENABLE
						LCD_1_Position(1, 10);
						LCD_1_PrCString(" syncd");
#endif

					// Error
					} else {
						fn = 0;			// Restart
						if (--retry_ctr < 0) {
							stat = JJY_STAT_ABORT;
//							bn = 9;
//							fn = 99;

							// Debug info
#ifdef LCD_ENABLE
							LCD_1_Position(1, 10);
							LCD_1_PrCString(" abort");
#endif
						}

						// Debug info
#ifdef LCD_ENABLE
						LCD_1_Position(0, 9);
						LCD_1_WriteData(retry_ctr + '0');
						if (!pa1) {
							LCD_1_Position(0, 11);
							LCD_1_WriteData('S');
						}
						if (!pa2) {
							LCD_1_Position(0, 10);
							LCD_1_WriteData('S');
						}
#endif
#ifdef LED_ENABLE
						I2C_LED60_DrawBG(30, retry_ctr << 1, 15);
#endif
					}

				}

			}
		}
	}

	//
	// Close user modules
	//
	if (stat == JJY_STAT_CANCEL) {
		stat = JJY_STAT_NORMAL;
	}
	if ((stat <= JJY_STAT_NORMAL) && init_flg) {
		LED_1_Stop();
		SAR6_1_Stop();
		AMD_CR0 &= 0b10001111;
		LPF2_1_Stop();
		ACB02CR1 &= 0b10111111;	
		PGA_2_Stop();
		BPF4_1_Stop();
		AMPINV_1_Stop();
		PGA_1_Stop();
		Counter16_2_Stop();
		Counter8_1_Stop();
		Counter16_1_Stop();
		AN_Power_Off();
		AN_Power_Stop();
		I2C_LED60_Dim(I2C_LED60_DimStart);
		I2C_LED7SEG_Dim(I2C_LED7SEG_DimStart);
		init_flg = 0;
	}
	
	return(stat);
}



//-----------------------------------------------------------------------------
//
//	M a i n
//
//-----------------------------------------------------------------------------
void main(void) {
	unsigned char	ss_b;
	unsigned char	hh_b;
	signed char		jjy_stat;
	unsigned char	sw_up_cnt;
	unsigned char	sw_down_cnt;
	unsigned char	sw_select_cnt;

	//
	// Init
	//
	M8C_EnableGInt;
	Timer16_1_Start();
	Timer16_1_EnableInt();
	LCD_1_Start();
	LCD_1_InitBG(LCD_1_SOLID_BG);					// Bar graph mode
	SW_Select_DataShadow |= SW_Select_MASK;			// High level - pullups
	SW_Select_Data_ADDR   = SW_Select_DataShadow;	//
	SW_Up_DataShadow     |= SW_Up_MASK;				// High level - pullups
	SW_Up_Data_ADDR       = SW_Up_DataShadow;		//
	SW_Down_DataShadow   |= SW_Down_MASK;			// High level - pullups
	SW_Down_Data_ADDR     = SW_Down_DataShadow;		//
	I2C_LED7SEG_Start();							// Private module
	I2C_LED60_Start();								// Private module
	
//	jjy_stat	= JJY_STAT_ADJUST;
	jjy_stat	= JJY_STAT_ABORT;
	ss_b		= 99;
	hh_b		= 99;
	LCD_1_Position(0,0);
	LCD_1_PrCString("Wall Clock 0.1");
	LCD_1_Position(1,0);
	LCD_1_PrCString("(c)2010T.Togashi");

	//
	// Event loop
	//
	for(;;) {
		
		//
		// Wait for event
		//
		for(;;) {
		
			//
			// Sleep until intrrupt
			//
			if (jjy_stat <= JJY_STAT_NORMAL) {
				BYTE ss;				// status save
				BYTE sa;				//
		
				sa = ABF_CR0;			// Analog Output Buffer Control Register 0
				ss = ARF_CR;			// Analog Reference Control Register
				
				ABF_CR0 = 0x00;			// block power off (~-230uA)
				ARF_CR  = 0x00;			// block power off (~-3mA)
				M8C_Sleep;				// clock stop      (~3uA)
				
				ARF_CR  = ss;			// dummy
				ARF_CR  = ss;			// power resume
				ABF_CR0 = sa;			// 
			} else {
				unsigned char i;
		
				// Minimum wait
				for(i = 0; i < 250; i++);
			}
			
			break;
		}
		
		//
		// Push switch task
		//
		if (!(SW_Up_Data_ADDR & SW_Up_MASK)) {		// Manual sync switch
			if (sw_up_cnt == 0) {					// First sense
				jjy_stat = JJY_STAT_ADJUST;			// 
			}
			sw_up_cnt++;
		} else {
			sw_up_cnt = 0;
		}
		if (!(SW_Down_Data_ADDR & SW_Down_MASK)) {	// Stop sync switch
			if (sw_down_cnt == 0) {					// First sense
				jjy_stat = JJY_STAT_CANCEL;
			}
			sw_down_cnt++;
		} else {
			sw_down_cnt = 0;
		}
		if (!(SW_Select_Data_ADDR & SW_Select_MASK)) {	// Manual set switch
			if (sw_select_cnt == 0) {					// First sense
				dateset();
				hh_b = 99;
			}
			sw_select_cnt++;
		} else {
			sw_select_cnt = 0;
		}

		//
		// Display task
		//
		if (ss_b != ss) {
			signed char	a;

			ss_b = ss;

			if (hh_b != hh) {
				hh_b = hh;
				
				if (jjy_stat == JJY_STAT_ABORT) {
					unsigned char i;
					
					t_sprintf(buf, "<Unsync>");
					
					for(i = 0; i < sizeof(msg01); i++) {
						I2C_LED7SEG_PutPattern(msg01[i], i);
					}
				} else {
					t_sprintf(buf, "%d/%02d/%02d", yy, mm, dd);
				
					I2C_LED7SEG_DispInt(yy+2000, I2C_LED7SEG_Digit5, 4);
					I2C_LED7SEG_DispInt(mm*100+dd, I2C_LED7SEG_Digit1, 4);
				}
				LCD_1_Position(0, 0);
				LCD_1_PrString(buf);
			}

			t_sprintf(buf, "%02d:%02d:%02d", hh, mi, ss);
			LCD_1_Position(1, 0);
			LCD_1_PrString(buf);
			
			a = (hh % 12) * 5;
			I2C_LED60_DrawBG(a-6,   0, -3);
			I2C_LED60_DrawBG(a-1,   3, -3);
			I2C_LED60_DrawBG(mi-1, -1, -2);
			I2C_LED60_DrawBG(ss-1, -1,  2);
		}

		//
		// JJY recieve task
		//
		if ((mi == 33) && (ss == 0)) {				// 3:33:00/day
			if ((hh == 3) || (jjy_stat == JJY_STAT_ABORT)) {
				jjy_stat = JJY_STAT_ADJUST;			// Resync
			}
		}
		if (jjy_stat > JJY_STAT_NORMAL) {
			static unsigned char	tick_b = 0;
			unsigned char			tick;

			// 1/8sec interval
			tick = Timer16_1_wReadTimer() >> 12;		// 32768/8=4096
			if (tick_b != tick) {
				tick_b = tick;
				
				// Body
				jjy_stat = recv_jjy(jjy_stat);
			}
		}

		//
		// Ambient bright control task
		//
		
		//
		// PIR sense task
		//
		
	}
}
