//
//  t_sprintf.c
//
//  stdio.h "sprintf" like print data formatter
//  Copyright (C) 2009-2010 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//	Update history
//	---------- ----- -------------------------------------------
//	2009.08.02 v0.0  First cording for ImageCraft C Compiler
//	2009.12.18 v0.0  bug fix, %hhd unsign
//	2010.02.20 v0.1  Support PIC18F & vararg (Reset update history)
//	2010.04.02 v0.1  Support PSoC1 ICC7.02 & vararg
//	2010.04.03 v0.1  bug fix, 10* -> *, -10* -> 10*
//

#include <string.h>
#include <stdlib.h>
#include <stdarg.h>


#if defined(_M8C) && !defined(HI_TECH_C)	// ADD 2010.04.02  Reduction stack area
#define SC	static							// ADD 2010.04.02  Storage class (auto or static)
#endif										// ADD 2010.04.02


// Number to string convert
//#if defined(HI_TECH_C) || defined(_M8C)	// DEL 2010.04.02 Reduction code memory
#if defined(HI_TECH_C)						// ADD 2010.04.02
#define t_ultoa(a,b,c)	ultoa(a,b,c)
#define t_ltoa(a,b,c)	ltoa(a,b,c)
#else
void t_ultoa(char *buf, unsigned long val, int base) {
	SC unsigned long	u;

	u = val;
//	while(u > base) {					// DEL 2010.04.03
	while(u >= base) {					// ADD 2010.04.03
		u = u / base;
		buf++;
	}
	*(buf+1) = '\0';

	u = val;
	do {
		unsigned char	m;

		m = u % base;
		u = u / base;
		if (m < 10) {
			*buf = m + '0';
		} else {
			*buf = m + 'a' - 10;
		}
		--buf;
	} while(u != 0);
	return;
}

void t_ltoa(char *buf, long val, int base) {
	if (val < 0) {
		*buf++ = '-';
		val = -val;
	}
	t_ultoa(buf, val, base);
	return;
}
#endif

#ifdef __18CXX
#define cstrncpy(s,d,n)	strncpypgm2ram(s,d,n)
#endif

#ifdef DEBUG
#define cstrncpy(s,d,n)	strncpy(s,d,n)
#endif


void t_sprintf(char *buf, const char *fmt, ...) {
	signed char n;						// argument#
	va_list	ap;

	va_start(ap, fmt);
	n = 0;
	while(*fmt != '\0') {

		if (*fmt != '%') {
			*buf++ = *fmt++;
		} else {
			char a;						// align
			char s;						// sign char
			char z;						// padding
			signed char w;				// width
			signed char p;				// precision
			char f;						// format char
			char t;						// data type
			SC char pf[5];				// piece of fmt  CHG 2010.04.02
			void *arg;					// argument

			fmt++;
			a = 0;
			s = '\0';
			z = ' ';
			w = 0;
			p = 0;
			f = '\0';
			t = '\0';

			//
			// Format analyze
			//

			// Get flag
			while(*fmt != '\0') {
				char c;

				c = *fmt;
				switch(c) {
				case '-':				// reverse align
					a = c;
					break;
				case '+':				// force sign +/-
					s = c;
					break;
				case ' ':               // force sign  /-
					s = c;
					break;
				case '#':				// N/A
					break;
				case '0':				// zero padding
					z = c;
					break;
				default:
					c = '\0';
					break;
				}
				if (c == '\0') {
					break;
				}
				fmt++;
			}

			// get fixed field width
			cstrncpy(pf, fmt, sizeof(pf)-1);
			pf[sizeof(pf)-1] = '\0';
			w = atoi(pf);
			while(isdigit(*fmt)) fmt++;

			// get precision field
			if (*fmt == '.') {
				fmt++;
				cstrncpy(pf, fmt, sizeof(pf)-1);
				pf[sizeof(pf)-1] = '\0';
				p = atoi(pf);
				while(isdigit(*fmt)) fmt++;
			}

			// get format charactor or data type
			while(*fmt != '\0') {
				f = *fmt++;
				if ((f == 'h') || (f == 'l') || (f == 'j') || (f == 'z') || (f == 't')) {
					if (t != '\0') {
						t = toupper(t);
					} else {
						t = f;
					}
				} else {
					break;
				}
			}

			//
			// Format out
			//
			switch(f) {
			case '%':
				*buf++ = f;
				break;
			case 'c':
				*buf++ = va_arg(ap, int);
				break;
			case 'd':
			case 'i':
			case 'u':
			case 'o':
			case 'x':
			case 's':
				{
					signed char l;		// data length
					SC char tb[16];		// number->str  CHG 2010.04.02
					char *ts;			// target str

					if (f == 's') {
						ts = va_arg(ap, char *);
					} else {
						unsigned char b;

						// radix
						switch(f) {
						case 'x':
							b = 16;
							break;
						case 'o':
							b = 8;
							break;
						default:
							b = 10;
							break;
						}

						// signed number
						if ((f == 'd') || (f == 'i')) {
							SC signed long d;

							switch(t) {
							case 'h':
								d = va_arg(ap, short);
								break;
							case 'H':
								d = va_arg(ap, signed char);
								break;
							case 'l':
							case 'L':
								d = va_arg(ap, long);
								break;
							default:
								d = va_arg(ap, int);
								break;
							}

							
							if ((s == '\0') || (d < 0)) {
								t_ltoa(tb, d, b);
							} else {	// +/-option
								tb[0] = s;
								t_ltoa(&tb[1], d, b);
							}

						// unsigned number
						} else {
							SC unsigned long u;

							switch(t) {
							case 'h':
								u = va_arg(ap, unsigned short);
								break;
							case 'H':
								u = va_arg(ap, unsigned char);
								break;
							case 'l':
							case 'L':
								u = va_arg(ap, unsigned long);
								break;
							default:
								u = va_arg(ap, unsigned int);
								break;
							}

							t_ultoa(tb, u, b);

						}

						ts = tb;
						a = !a;
					}

					l = strlen(ts);
					if (w == 0) {
						w = l;
					}

					if (a) {				// Right align
						signed char i;
					
						i = w - l;
						if (i < 0) {		// s is longer
							ts += (-i + 1);	// skip char's
							*buf++ = '~';	// overflow marking
							--w;
							l = w;
						}
						for(; i > 0; --i, --w) {
							*buf++ = z;		// Before padding
						}
					}
					for(; *ts != '\0' && w > 0; ts++, --w) {
						*buf++ = *ts;
					}
					if (*ts != '\0') {
						*(buf-1) = '~';		// overflow marking
					}
					for(; w > 0; --w) {		// Left align
						*buf++ = z;			// After padding
					}
				}
				break;
			default:
				*buf++ = f;
				break;
			}
		}
	}
	va_end(ap);
	*buf = '\0';
	return;
}

#ifdef DEBUG
#include <stdio.h>
main(){
	char buf[100];

	t_sprintf(buf, "<%lx>", -1234567L);
	printf("%s\n", buf);
}
#endif
